import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ElementRef, HostBinding, HostListener, Input, TemplateRef, ViewChild } from "@angular/core";
import { debounceTime, distinctUntilChanged } from "rxjs/operators";
import { Subject } from "rxjs/Subject";
import { Subscription } from "rxjs/Subscription";
import { DataType } from "../data-operations/data-util";
import { BOOLEAN_FILTERS, DATE_FILTERS, NUMBER_FILTERS, STRING_FILTERS } from "../data-operations/filtering-condition";
import { IgxToggleDirective } from "../directives/toggle/toggle.directive";
import { IgxGridAPIService } from "./api.service";
var IgxGridFilterComponent = /** @class */ (function () {
    function IgxGridFilterComponent(gridAPI, cdr, elementRef) {
        var _this = this;
        this.gridAPI = gridAPI;
        this.cdr = cdr;
        this.elementRef = elementRef;
        this.dialogShowing = false;
        this.dialogPosition = "igx-filtering__options--to-right";
        this.UNARY_CONDITIONS = [
            "true", "false", "null", "notNull", "empty", "notEmpty",
            "yesterday", "today", "thisMonth", "lastMonth", "nextMonth",
            "thisYear", "lastYear", "nextYear"
        ];
        this.filterChanged = new Subject();
        this.chunkLoaded = new Subscription();
        this.MINIMUM_VIABLE_SIZE = 240;
        this.filterChanged.pipe(debounceTime(250), distinctUntilChanged()).subscribe(function (value) { return _this.value = value; });
    }
    Object.defineProperty(IgxGridFilterComponent.prototype, "value", {
        get: function () {
            return this._value || "";
        },
        set: function (val) {
            if (!val) {
                this.clearFiltering();
                return;
            }
            this._value = this.transformValue(val);
            this.filter();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "dataType", {
        get: function () {
            return this.column.dataType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "conditions", {
        get: function () {
            var conditions = [];
            switch (this.dataType) {
                case DataType.String:
                    conditions = Object.keys(STRING_FILTERS);
                    break;
                case DataType.Number:
                    conditions = Object.keys(NUMBER_FILTERS);
                    break;
                case DataType.Boolean:
                    conditions = Object.keys(BOOLEAN_FILTERS);
                    break;
                case DataType.Date:
                    conditions = Object.keys(DATE_FILTERS);
            }
            return conditions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "template", {
        get: function () {
            switch (this.dataType) {
                case DataType.String:
                case DataType.Number:
                    return this.defaultFilterUI;
                case DataType.Date:
                    return this.defaultDateUI;
                case DataType.Boolean:
                    return null;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "filterCSS", {
        get: function () {
            if (this.dialogShowing) {
                return "igx-filtering__toggle--active";
            }
            if (this.filteringExpression()) {
                return "igx-filtering__toggle--filtered";
            }
            return "igx-filtering__toggle";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "gridID", {
        get: function () {
            return this.column.gridID;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridFilterComponent.prototype, "styleClasses", {
        get: function () {
            return "igx-filtering";
        },
        enumerable: true,
        configurable: true
    });
    IgxGridFilterComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.chunkLoaded = this.gridAPI.get(this.gridID).headerContainer.onChunkPreload.subscribe(function () {
            if (!_this.toggleDirective.collapsed) {
                _this.toggleDirective.collapsed = true;
                _this.refresh();
            }
        });
    };
    IgxGridFilterComponent.prototype.ngOnDestroy = function () {
        this.filterChanged.unsubscribe();
        this.chunkLoaded.unsubscribe();
    };
    IgxGridFilterComponent.prototype.refresh = function () {
        this.dialogShowing = !this.dialogShowing;
        this.cdr.detectChanges();
    };
    IgxGridFilterComponent.prototype.isActive = function (value) {
        return this._filterCondition === value;
    };
    Object.defineProperty(IgxGridFilterComponent.prototype, "unaryCondition", {
        get: function () {
            for (var _i = 0, _a = this.UNARY_CONDITIONS; _i < _a.length; _i++) {
                var each = _a[_i];
                if (this._filterCondition && this._filterCondition === each) {
                    return true;
                }
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridFilterComponent.prototype.filter = function () {
        var grid = this.gridAPI.get(this.gridID);
        this.gridAPI.filter(this.column.gridID, this.column.field, this._value, this.column.filteringCondition, this.column.filteringIgnoreCase);
        grid.onFilteringDone.emit({
            expression: {
                fieldName: this.column.field,
                filteringCondition: this.column.filteringCondition,
                filteringIgnoreCase: this.column.filteringIgnoreCase,
                searchVal: this._value
            }
        });
        this.cdr.markForCheck();
    };
    IgxGridFilterComponent.prototype.clearFiltering = function () {
        this._value = null;
        this._filterCondition = undefined;
        this.gridAPI.clear_filter(this.gridID, this.column.field);
        this.cdr.markForCheck();
    };
    IgxGridFilterComponent.prototype.conditionChanged = function (value) {
        this._filterCondition = value;
        if (this.dataType === DataType.String) {
            this.column.filteringCondition = STRING_FILTERS[value];
        }
        else if (this.dataType === DataType.Number) {
            this.column.filteringCondition = NUMBER_FILTERS[value];
        }
        else if (this.dataType === DataType.Boolean) {
            this.column.filteringCondition = BOOLEAN_FILTERS[value];
        }
        else if (this.dataType === DataType.Date) {
            this.column.filteringCondition = DATE_FILTERS[value];
        }
        this.filter();
    };
    IgxGridFilterComponent.prototype.onInputChanged = function (val) {
        this.filterChanged.next(val);
    };
    Object.defineProperty(IgxGridFilterComponent.prototype, "disabled", {
        get: function () {
            if (this.value && !this.unaryCondition) {
                return false;
            }
            else if (this.unaryCondition) {
                return false;
            }
            return true;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridFilterComponent.prototype.onClick = function (event) {
        event.stopPropagation();
        var grid = this.gridAPI.get(this.gridID);
        var gridRect = grid.nativeElement.getBoundingClientRect();
        var dropdownRect = this.elementRef.nativeElement.getBoundingClientRect();
        var x = dropdownRect.left;
        var x1 = gridRect.left + gridRect.width;
        x += window.pageXOffset;
        x1 += window.pageXOffset;
        if (Math.abs(x - x1) < this.MINIMUM_VIABLE_SIZE) {
            this.dialogPosition = "igx-filtering__options--to-left";
        }
    };
    IgxGridFilterComponent.prototype.transformValue = function (value) {
        if (this.dataType === DataType.Number) {
            value = parseInt(value, 10);
        }
        else if (this.dataType === DataType.Boolean) {
            value = Boolean(value);
        }
        else if (this.dataType === DataType.Date) {
            value = new Date(Date.parse(value));
        }
        return value;
    };
    IgxGridFilterComponent.prototype.filteringExpression = function () {
        var _this = this;
        var expr = this.gridAPI.get(this.gridID)
            .filteringExpressions.find(function (x) { return x.fieldName === _this.column.field; });
        if (expr) {
            this._value = expr.searchVal;
            this._filterCondition = expr.condition.name;
            if (!this.unaryCondition && !this._value) {
                return false;
            }
            return true;
        }
        else {
            this._value = null;
            this._filterCondition = undefined;
        }
    };
    IgxGridFilterComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid-filter",
                    template: "<ng-template #defaultFilterUI>     <div *ngIf=\"!unaryCondition\" class=\"igx-filtering__op\">         <input type=\"text\" igxInput placeholder=\"value\" autocomplete=\"off\" [value]=\"value\" (input)=\"onInputChanged($event.target.value)\"         />     </div>     <br> </ng-template>  <ng-template #defaultDateUI>     <igx-datePicker *ngIf=\"!unaryCondition\" [(ngModel)]=\"value\"></igx-datePicker> </ng-template>  <div class=\"igx-filtering\">     <div [attr.class]=\"filterCSS\">         <span class=\"toggle-icon\" [igxToggleAction]=\"directive\">             <igx-icon fontSet=\"material\" name=\"filter_list\"></igx-icon>         </span>     </div> </div>  <span igxToggle (onOpen)=\"refresh()\" (onClose)=\"refresh()\" #directive=\"toggle\" [attr.class]=\"dialogPosition\">     <div>         <select (change)=\"conditionChanged($event.target.value)\">             <option [selected]=\"isActive(each)\" *ngFor=\"let each of conditions\" [value]=\"each\">{{ each | filterCondition | titlecase }}</option>         </select>     </div>     <ng-container *ngTemplateOutlet=\"template; context: { $implicit: this }\"></ng-container>     <div class=\"igx-filtering__options-bgroup\">         <button igxButton igxRipple (click)=\"clearFiltering()\" [disabled]=\"disabled\">Reset</button>         <button igxButton igxRipple (click)=\"filter()\" [disabled]=\"disabled\">Filter</button>     </div> </span>"
                },] },
    ];
    /** @nocollapse */
    IgxGridFilterComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: ChangeDetectorRef, },
        { type: ElementRef, },
    ]; };
    IgxGridFilterComponent.propDecorators = {
        "column": [{ type: Input },],
        "styleClasses": [{ type: HostBinding, args: ["class",] },],
        "defaultFilterUI": [{ type: ViewChild, args: ["defaultFilterUI", { read: TemplateRef },] },],
        "defaultDateUI": [{ type: ViewChild, args: ["defaultDateUI", { read: TemplateRef },] },],
        "toggleDirective": [{ type: ViewChild, args: [IgxToggleDirective, { read: IgxToggleDirective },] },],
        "onClick": [{ type: HostListener, args: ["click", ["$event"],] },],
    };
    return IgxGridFilterComponent;
}());
export { IgxGridFilterComponent };
//# sourceMappingURL=grid-filtering.component.js.map